"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CustomizedLanguageService = void 0;
const constants_1 = require("./constants");
const types_1 = require("./types");
const utils_1 = require("./utils");
class CustomizedLanguageService {
    constructor(info, typescript, logger) {
        this.info = info;
        this.typescript = typescript;
        this.logger = logger;
    }
    getApplicableRefactors(fileName, positionOrRange, preferences) {
        const context = this.getRefactorContext(fileName);
        if (!context) {
            this.logger.log("Cannot construct refactor context");
            return [];
        }
        const ts = this.typescript;
        const { file } = context;
        const targetInfo = this.getTargetInfo(file, utils_1.getPositionOfPositionOrRange(positionOrRange));
        if (targetInfo.type !== types_1.OkType.Ok) {
            this.logger.log(targetInfo.reason);
            if (!(preferences === null || preferences === void 0 ? void 0 : preferences.provideRefactorNotApplicableReason) ||
                !targetInfo.extra) {
                return [];
            }
            const [actionName, actionDescription] = utils_1.kindToActionNameAndDesc(targetInfo.extra, ts);
            return [
                {
                    name: constants_1.refactorName,
                    description: constants_1.refactorDescriptions,
                    actions: [
                        {
                            name: actionName,
                            description: actionDescription,
                            notApplicableReason: targetInfo.reason
                        }
                    ]
                }
            ];
        }
        const [actionName, actionDescription] = utils_1.kindToActionNameAndDesc(targetInfo.value.kind, ts);
        return [
            {
                name: constants_1.refactorName,
                description: constants_1.refactorDescriptions,
                actions: [
                    {
                        name: actionName,
                        description: actionDescription
                    }
                ]
            }
        ];
    }
    getEditsForRefactor(fileName, formatOptions, positionOrRange, refactor, actionName, preferences) {
        if (refactor !== constants_1.refactorName ||
            (actionName !== constants_1.convertEnumIntoStringLiteralActionName &&
                actionName !== constants_1.convertEnumMemberIntoStringLiteralActionName)) {
            return undefined;
        }
        const context = this.getRefactorContext(fileName);
        if (!context) {
            this.logger.log("Cannot construct refactor context");
            return undefined;
        }
        const ts = this.typescript;
        const { file } = context;
        const targetInfo = this.getTargetInfo(file, utils_1.getPositionOfPositionOrRange(positionOrRange));
        if (targetInfo.type !== types_1.OkType.Ok) {
            this.logger.log(targetInfo.reason);
            return undefined;
        }
        return this.doInTextChanges(formatOptions, preferences, changeTracker => {
            if (ts.isEnumDeclaration(targetInfo.value)) {
                this.doChangesForEnumDeclaration(file, targetInfo.value, changeTracker);
            }
            else {
                this.doChangesForEnumMember(file, targetInfo.value, changeTracker);
            }
        });
    }
    doInTextChanges(formatOptions, preferences, cb) {
        const formatContext = this.typescript.formatting.getFormatContext(formatOptions);
        const textChangesContext = {
            formatContext,
            host: this.info.languageServiceHost,
            preferences: preferences || {}
        };
        const edits = this.typescript.textChanges.ChangeTracker.with(textChangesContext, cb);
        return {
            edits
        };
    }
    getTargetInfo(file, pos) {
        const ts = this.typescript;
        if (file.isDeclarationFile) {
            return types_1.Err("Cannot convert in d.ts files");
        }
        const currentToken = ts.getTokenAtPosition(file, pos);
        if (!ts.isStringLiteral(currentToken) &&
            !ts.isIdentifier(currentToken)) {
            return types_1.Err("Cannot find string or identifier in selection");
        }
        if ((!ts.isEnumDeclaration(currentToken.parent) &&
            !ts.isEnumMember(currentToken.parent)) ||
            currentToken.parent.name !== currentToken) {
            return types_1.Err("Cannot find enum declaration in selection");
        }
        if (ts.isEnumDeclaration(currentToken.parent)) {
            if (!currentToken.parent.members.some(utils_1.isConvertibleEnumMember)) {
                return types_1.Err("Enum declaration cannot be convert", currentToken.parent.kind);
            }
            return types_1.Ok(currentToken.parent);
        }
        else {
            if (!utils_1.isConvertibleEnumMember(currentToken.parent)) {
                return types_1.Err("Enum member cannot be convert", currentToken.parent.kind);
            }
            return types_1.Ok(currentToken.parent);
        }
    }
    doChangesForEnumDeclaration(file, enumDecl, changeTracker) {
        enumDecl.members
            .filter(utils_1.isConvertibleEnumMember)
            .forEach(member => this.doChangesForEnumMember(file, member, changeTracker));
    }
    doChangesForEnumMember(file, enumMember, changeTracker) {
        const ts = this.typescript;
        utils_1.assert(!enumMember.initializer, "Enum member cannot have initializer");
        utils_1.assert(ts.isIdentifier(enumMember.name) ||
            ts.isStringLiteral(enumMember.name), "Name of Enum member can only be identifier or string literal");
        changeTracker.replaceNode(file, enumMember, ts.factory.updateEnumMember(enumMember, enumMember.name, ts.factory.createStringLiteral(enumMember.name.text)));
    }
    getRefactorContext(fileName) {
        const program = this.info.languageService.getProgram();
        if (!program) {
            this.logger.log("Cannot find program");
            return undefined;
        }
        const file = program.getSourceFile(fileName);
        if (!file) {
            this.logger.log("Cannot find source file");
            return undefined;
        }
        return {
            file,
            program
        };
    }
}
exports.CustomizedLanguageService = CustomizedLanguageService;
//# sourceMappingURL=service.js.map